#!/bin/bash

###############################################################################
printf -- '-%.0s' {1..80}
echo
echo "build-docker-compose-file.sh: Tool to bulid docker-compose.yaml file"
printf -- '-%.0s' {1..80}
echo

###############################################################################
# Remove tmp directory and recreate one for storing temp files
rm -rf ./tmp
mkdir ./tmp

###############################################################################
# Define functions for building docker containers

# Build a proxy docker container for the give proxy.
#
# Usage:
#   `build_proxy_docker host port username password`
build_proxy_docker() {
    host="$1"
    port="$2"
    username="$3"
    password="$4"
    block_domains_cmds="$5"
    outname="./tmp/tun${host}.yaml"
    echo -e "${GREEN} BUILD PROXY DOCEKR: $host:$port ${NOCOLOUR}" >&2

    sed -e "s/{{protocol}}/http/g" \
        -e "s/{{host}}/$host/g" \
        -e "s/{{port}}/$port/g" \
        -e "s/{{username}}/$username/g" \
        -e "s/{{password}}/$password/g" \
        -e "s/{{block_domains_cmds}}/$block_domains_cmds/g" \
        ./templates/tun2socks_template.yaml >"$outname"

    # return the created temp file
    echo "$outname"
}

# Build packetshare docker container that "links" to the corresponding
# proxy docker via the network.
#
# The idea is that, the whole trafic to this docker container will be
# redirected through the proxy container, so that packetshare recogines
# the public IP of the running instance as the IP of the proxy.
#
# Usage:
#   `build_packetshare_docker proxy_host packetshare_email packetshare_password`
build_packetshare_docker() {
    proxy_host="$1"
    packetshare_email="$2"
    packetshare_password="$3"
    outname="./tmp/packetshare${proxy_host}.yaml"

    echo -e "${GREEN} BUILD PACKETSHARE DOCKER: $packetshare_email runing on host $proxy_host ${NOCOLOUR}" >&2
    sed -e "s/{{proxy_host}}/$proxy_host/g" \
        -e "s/{{packetshare_email}}/$packetshare_email/g" \
        -e "s/{{packetshare_password}}/$packetshare_password/g" \
        ./templates/packetshare_template.yaml >"$outname"

    # return the created temp file
    echo "$outname"
}

# Build one proxy container and one packetshare container for each proxy.
#
# The definition of these 2 containers are placed together in a docker
# compose file, which will be run afterward to create docker containers.
#
# Usage:
#   `build_docker_compose_file proxy_file packetshare_email packetshare_password block_domains_cmds`
build_docker_compose_file() {
    compose_file="./docker-compose.yaml"

    # Remove existing file if exists
    rm -rf $compose_file

    # Prepare file structure
    echo -e "name: nga-proxy-linux-packetshare\n" >>$compose_file
    echo -e "services:\n" >>$compose_file

    # Input params
    proxy_file="$1"
    packetshare_email="$2"
    packetshare_password="$3"
    block_domains_cmds="$4"

    if [ ! -f "$proxy_file" ]; then
        echo -e "${RED}Proxy file $proxy_file does not exist ${NOCOLOUR}" >&2
        exit 1
    fi

    # Remove special characters from proxy file
    sed -i 's/\r//g' "$proxy_file"

    # Read proxy file line-by-line and build one container for each proxy
    while IFS= read -r line || [[ -n "$line" ]]; do
        # Split proxy text string into params
        IFS=':' read -r -a proxy_params <<<"$line"

        proxy_tmp_file=$(
            build_proxy_docker "${proxy_params[0]}" "${proxy_params[1]}" "${proxy_params[2]}" "${proxy_params[3]}" "$block_domains_cmds"
        )
        packetshare_tmp_file=$(
            build_packetshare_docker "${proxy_params[0]}" "$packetshare_email" "$packetshare_password"
        )

        cat "$proxy_tmp_file" >>$compose_file
        cat "$packetshare_tmp_file" >>$compose_file

        # Clean temp files
        rm -rf "$proxy_tmp_file"
        rm -rf "$packetshare_tmp_file"

    done <"$proxy_file"

    # Return the docker compose file name
    echo $compose_file
}
